package com.company.example.app;

import java.io.File;
import java.time.OffsetDateTime;
import java.util.Objects;
import java.util.UUID;

import com.company.example.entity.*;
import com.vaadin.flow.component.upload.Receiver;
import io.jmix.core.*;
import io.jmix.core.security.CurrentAuthentication;
import io.jmix.flowui.component.upload.receiver.FileTemporaryStorageBuffer;
import io.jmix.flowui.upload.TemporaryStorage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component("trz_RefersFileService")
public class RefersFileService {

    private static final Logger log = LoggerFactory.getLogger(RefersFileService.class);

    private final DataManager dataManager;
    private final TemporaryStorage temporaryStorage;
    private final FileStorage fileStorage;
    private final CurrentAuthentication currentAuthentication;

    @Autowired
    public RefersFileService(DataManager dataManager, TemporaryStorage temporaryStorage, FileStorage fileStorage, CurrentAuthentication currentAuthentication) {
        this.dataManager = dataManager;
        this.temporaryStorage = temporaryStorage;
        this.fileStorage = fileStorage;
        this.currentAuthentication = currentAuthentication;
    }

    public void removeRefersFile(FileRef fileRef, UUID refersKey) {
        log.info("Removing refers file: {} : {}", fileRef, refersKey);
        if (Objects.nonNull(fileRef) && Objects.nonNull(refersKey)) {
            dataManager.remove(findById(refersKey));
            fileStorage.removeFile(fileRef);
        }
    }

    public RefersFileView saveRefersFile(Receiver receiver, UUID refersKey) {
        log.info("Saving refers file: {} : {}", receiver, refersKey);
        FileTemporaryStorageBuffer buffer = (FileTemporaryStorageBuffer) receiver;
        RefersFileView refersFileView = dataManager.create(RefersFileView.class);
        if (Objects.nonNull(buffer)) {
            UUID fileId = buffer.getFileData().getFileInfo().getId();
            File file = temporaryStorage.getFile(fileId);
            if (Objects.nonNull(file)) {
                String fileName = fileName(buffer.getFileData().getFileName());
                Long size = buffer.getFileData().getFileInfo().getFile().length();
                FileRef fileRef = temporaryStorage.putFileIntoStorage(fileId, buffer.getFileData().getFileName());
                temporaryStorage.deleteFile(fileId);
                refersFileView.setFileId(fileId);
                refersFileView.setRefersFile(saveRefersFile(fileRef, fileName, size, refersKey));
                refersFileView.setFileRef(fileRef);
            }
        }
        return refersFileView;
    }

    public RefersFile saveRefersFile(FileRef fileRef, String fileName, Long size, UUID refersKey) {
        log.info("Saving file: {} : {}", fileRef, refersKey);
        RefersFile refersFile = findById(refersKey);
        if (Objects.nonNull(refersFile)) {
            fileStorage.removeFile(refersFile.getFileRef());
            dataManager.remove(refersFile);
        }

        refersFile = dataManager.create(RefersFile.class);
        refersFile.setFileRef(fileRef);
        refersFile.setName(fileName);
        refersFile.setSize(size);
        refersFile.setCreatedBy(getCurrentUser().getUsername());
        refersFile.setCreatedDate(OffsetDateTime.now());

        log.info("File created: {} : {}", refersFile, fileRef);
        return dataManager.save(refersFile);
    }

    public RefersFile findById(UUID id) {
        if (Objects.isNull(id)) return null;
        return dataManager.load(RefersFile.class).id(id).one();
    }

    private String fileName(String name) {
        return name.contains(".") ? name.substring(0, name.lastIndexOf(".")) : name;
    }

    private User getCurrentUser() {
        return (User) currentAuthentication.getUser();
    }
}