package com.company.example.view.main;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.Objects;

import com.company.example.entity.UseAgreement;
import com.company.example.exceptions.ResourceException;
import com.vaadin.flow.component.html.IFrame;
import com.vaadin.flow.component.notification.NotificationVariant;
import com.vaadin.flow.router.Route;
import com.vaadin.flow.server.StreamResource;
import com.vaadin.flow.server.VaadinSession;
import io.jmix.core.DataManager;
import io.jmix.core.FileStorage;
import io.jmix.core.querycondition.PropertyCondition;
import io.jmix.flowui.*;
import io.jmix.flowui.action.DialogAction;
import io.jmix.flowui.app.main.StandardMainView;
import io.jmix.flowui.kit.component.button.JmixButton;
import io.jmix.flowui.view.*;
import lombok.RequiredArgsConstructor;

@Route("")
@ViewController("trz_MainView")
@ViewDescriptor("main-view.xml")
@RequiredArgsConstructor
public class MainView extends StandardMainView {

    private final Dialogs dialogs;
    private final FileStorage fileStorage;
    private final UiComponents components;
    private final DataManager dataManager;
    private final Notifications notifications;

    @ViewComponent
    private JmixButton logoutButton;

    @Subscribe
    public void onReady(final ReadyEvent event) {
        var useAgreements = getCurrentLicenseOfUse();
        if (Objects.nonNull(useAgreements)) {
            var fileRef = useAgreements.getAgreement().getFileRef();
            try (var inputStream = fileStorage.openStream(fileRef)) {
                var byteArrayInputStream = new ByteArrayInputStream(inputStream.readAllBytes());
                var resource = new StreamResource(fileRef.getFileName(), () -> byteArrayInputStream);
                resource.setContentType("application/pdf");
                var registration = VaadinSession.getCurrent().getResourceRegistry().registerResource(resource);

                var frame = components.create(IFrame.class);
                frame.setWidth("100%");
                frame.getStyle().set("border", "unset");
                frame.setSrc(registration.getResourceUri().getPath());

                dialogs.createOptionDialog()
                        .withHeader("License Agreement")
                        .withContent(frame)
                        .withActions(
                                new DialogAction(DialogAction.Type.NO)
                                        .withHandler(e -> logoutButton.click()),
                                new DialogAction(DialogAction.Type.YES)
                                        .withHandler(e -> notifications.create("OK pressed")
                                                .withThemeVariant(NotificationVariant.LUMO_SUCCESS)
                                                .show())
                        )
                        .withWidth("45em")
                        .withHeight("55em")
                        .open();
            } catch (IOException ex) {
                throw new ResourceException("An error occurred while trying to read the resource: {}", ex);
            }
        }
    }

    private UseAgreement getCurrentLicenseOfUse() {
        var useAgreements = dataManager.load(UseAgreement.class)
                .condition(PropertyCondition.equal("hidden", false))
                //.fetchPlan("license-of-use-fetch-plan")
                .list();
        return useAgreements.isEmpty() ? null : useAgreements.getFirst();
    }
}
